@@ -7,4 +7,11 @@ mod tests {
         assert_eq!(2usize.next_power_of_two(), 2);
         assert_eq!(3usize.next_power_of_two(), 4);
     }
+
+    #[test]
+    fn negative_division() {
+        assert_eq!(-5 % 2, -1);
+        assert_eq!(-5 % -2, -1);
+        assert_eq!(5 % -2, 1);
+    }
 }
